import asyncio
import json
import logging
import os
from urllib.parse import urlparse

import aiohttp
import async_timeout
from botocore.auth import SigV4Auth
from botocore.awsrequest import AWSRequest
from botocore.session import Session

_LOGGER = logging.getLogger(__name__)

CREDS = Session().get_credentials()
LAMBDA_ENDPOINT_BASE = "https://lambda.eu-west-1.amazonaws.com/2015-03-31/functions"
LAMBDA_TIMEOUT = 120


def create_signed_headers(url, payload):
    host_segments = urlparse(url).netloc.split(".")
    service = host_segments[0]
    region = host_segments[1]
    request = AWSRequest(method="POST", url=url, data=payload)
    SigV4Auth(CREDS, service, region).add_auth(request)
    return dict(request.headers.items())


def log(msg, retry):
    if retry > 1:
        _LOGGER.warning(msg)
        return
    _LOGGER.error(msg)


async def invoke(
    func_name, payload, aiohttp_session, retries=3, timeout=LAMBDA_TIMEOUT
):
    """Used to invoke lambda functions async."""
    url = os.path.join(LAMBDA_ENDPOINT_BASE, func_name, "invocations")
    data = json.dumps(payload)
    signed_headers = create_signed_headers(url, data)

    for retry in range(retries, 0, -1):
        try:
            with async_timeout.timeout(timeout):
                try:
                    async with aiohttp_session.post(
                        url, data=data, headers=signed_headers
                    ) as response:
                        if response.status != 200:
                            msg = await response.json()
                            log(
                                f"Error getting data from {func_name}, resp code: {response.status}, {msg}",
                                retry,
                            )
                            continue
                        return await response.json()
                except aiohttp.client_exceptions.ClientConnectorError:
                    log("ClientConnectorError", retry)
                    continue
        except asyncio.TimeoutError:
            _LOGGER.error("Timed out %s", func_name)
            return {}

    _LOGGER.error(
        "Error getting data from %s. %s", func_name, payload.get("deviceId", "")
    )
    return {}



import base64
import json
import logging

import boto3
from botocore.exceptions import ClientError, NoCredentialsError

_LOGGER = logging.getLogger(__name__)

DEFAULT_REGION_NAME = "eu-west-1"


def get_secret_parser(secret_name, region_name=DEFAULT_REGION_NAME):
    try:
        secret = get_secret(secret_name, region_name)
    except (ClientError, NoCredentialsError):
        _LOGGER.error("No credentials")
        return {}
    if secret is None:
        return {}
    return json.loads(secret)


def get_secret(secret_name, region_name=DEFAULT_REGION_NAME):

    # Create a Secrets Manager client
    session = boto3.session.Session()
    client = session.client(service_name="secretsmanager", region_name=region_name)

    get_secret_value_response = client.get_secret_value(SecretId=secret_name)
    # Decrypts secret using the associated KMS CMK.
    # Depending on whether the secret is a string or binary, one of these fields will be populated.
    if "SecretString" in get_secret_value_response:
        return get_secret_value_response["SecretString"]
    return base64.b64decode(get_secret_value_response["SecretBinary"])



try:
    from contextlib import AsyncExitStack
except ImportError:  # To support python 3.6
    from async_exit_stack import AsyncExitStack

import asyncio
import logging

import aiobotocore

_LOGGER = logging.getLogger(__name__)


def get_aiosession():
    return aiobotocore.get_session()


class AwsBase:
    def __init__(self, service_name, region_name="eu-west-1") -> None:
        self._client = None
        self._context_stack = AsyncExitStack()
        self._region_name = region_name
        self._service_name = service_name
        self._init_lock = asyncio.Lock()

    async def close(self) -> None:
        await self._context_stack.aclose()
        self._client = None

    async def init_client_if_required(self, session=None) -> None:
        async with self._init_lock:
            if self._client is not None:
                return
            session = session or get_aiosession()
            self._client = await self._context_stack.enter_async_context(
                session.create_client(self._service_name, region_name=self._region_name)
            )



from setuptools import setup

install_requires = [
    "aioboto3",
    "aiohttp",
    "async_timeout",
    "async_exit_stack; python_version<'3.7'"
]


setup(
    name="tibber_aws",
    packages=["tibber_aws"],
    install_requires=install_requires,
    version="0.9.3",
    description="A python3 library to communicate with Aws",
    python_requires=">=3.5.3",
    author="Tibber",
    author_email="drdaniel@tibber.com",
    url="https://github.com/tibbercom/tibber-pyAws",
    license="MIT",
    classifiers=[
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
